#!/usr/bin/env python3

from scanner import scanner, download

# Test on scanner.py

# Try getting /a/ catalog as json
catalog_json = scanner.get_catalog_json("a", "4chan")

# Try getting a list of threads
list_of_threads = scanner.scan_thread("anime", catalog_json)
for i in list_of_threads:
    print(i)

# try writting a test log file
scanner.add_to_downloaded("139294614", "log", ".")

# test on download.py

# Try getting a web page text
download.load("http://www.4chan.org")



#!/usr/bin/env python3


def get_chan_info(chan):
    if chan == "4chan":
        # 4chan informations
        chan_base_url = "http://a.4cdn.org/"
        chan_image_base_url = "http://i.4cdn.org/"
        chan_thread_subfolder = "/res/"
        chan_image_subfolder = "/"
    elif chan == "lainchan":
        # lainchan informations (lamba board does not work yet)
        chan_base_url = "https://lainchan.org/"
        chan_image_base_url = "https://lainchan.org/"
        chan_thread_subfolder = "/res/"
        chan_image_subfolder = "/src/"
    elif chan == "uboachan":
        # uboachan informations
        chan_base_url = "https://uboachan.net/"
        chan_image_base_url = "https://uboachan.net/"
        chan_thread_subfolder = "/res/"
        chan_image_subfolder = "/src/"
    else:
        return False

    return [chan_base_url, chan_thread_subfolder, chan_image_subfolder,
            chan_image_base_url]



#!/usr/bin/env python3

import argparse
from scanner import chan_info
import json
import logging
import os
import sys
import re
import time
import urllib
import http.client
import requests
import threading


def load(url):
    response = requests.get(url)
    response.raise_for_status()
    return response.text


def add_to_downloaded_log(img_filename, tmp_log):
    f = open(tmp_log, "a")
    f.write("{0}\n".format(img_filename))
    f.close()


def was_downloaded(img_filename, tmp_log):
    if os.path.isfile(tmp_log):
        f = open(tmp_log, "r")
        if str(img_filename) in f.read():
            f.close()
            return True
        else:
            return False
    else:
        return False


def extension_condition(condition_ext, post_ext):
    if condition_ext:
        for extension in condition_ext:
            if extension == post_ext:
                return True
    else:
        # Always return true if condition was not specified
        return True

    return False


def width_condition(condition_width, post_width):
    if condition_width:
        if condition_width[0] == "=":
            if int(post_width) == int(condition_width.split("=")[-1]):
                return True
        elif condition_width[0] == "<":
            if int(post_width) < int(condition_width.split("<")[-1]):
                return True
        elif condition_width[0] == ">":
            if int(post_width) > int(condition_width.split(">")[-1]):
                return True
        else:
            print("width need to be in this format:")
            print(">1024, <256 or =1920")
            exit(1)
    else:
        # Always return true if condition was not specified
        return True

    return False


def height_condition(condition_height, post_height):
    if condition_height:
        if condition_height[0] == "=":
            if int(post_height) == int(condition_height.split("=")[-1]):
                return True
        elif condition_height[0] == "<":
            if int(post_height) < int(condition_height.split("<")[-1]):
                return True
        elif condition_height[0] == ">":
            if int(post_height) > int(condition_height.split(">")[-1]):
                return True
        else:
            print("height need to be in this format:")
            print(">1024, <256 or =1080")
            exit(1)
    else:
        # Always return true if condition was not specified
        return True

    return False


# Check if all condition returned true
def all_condition_check(condition_list):
    for i in condition_list:
        if not i:
            return False
    return True


def download_thread(thread_nb, board, chan, output_folder, folder, is_quiet, condition):

    # Getting info about the chan URL
    chan_url_info = chan_info.get_chan_info(chan)
    if not chan_url_info:
        print("{0} does not exist or is not supported.".format(chan))
        exit(1)
    base_url = chan_url_info[0]
    image_url = chan_url_info[3]
    thread_subfolder = chan_url_info[1]
    image_subfolder = chan_url_info[2]

    thread_url = "{0}{1}{2}{3}.json".format(base_url,
                                            board,
                                            thread_subfolder,
                                            thread_nb)
    image_url = "{0}{1}{2}".format(image_url, board, image_subfolder)

    tmp_log = ("/tmp/4scanner_tmp_{0}_{1}"
               .format(os.getpid(), threading.current_thread().name))

    directory = os.path.join(output_folder, 'downloads', chan, board,
                             folder + "/")
    if not os.path.exists(directory):
        try:
            os.makedirs(directory)
        except OSError as e:  # folder may have been created by other threads
            if e.errno != 17:
                raise
            pass

    while True:
        try:
            thread_json = json.loads(load(thread_url))
            for post in thread_json["posts"]:
                if 'filename' in post:
                    if not was_downloaded(post["tim"], tmp_log):
                        condition_list = []
                        condition_list.append(extension_condition(condition["ext"], post['ext']))
                        condition_list.append(width_condition(condition["width"], post['w']))
                        condition_list.append(height_condition(condition["height"], post['h']))
                        download_img = all_condition_check(condition_list)

                        if download_img:
                            try:
                                pic_url = "{0}{1}{2}".format(image_url,
                                                             post["tim"],
                                                             post["ext"])
                                out_pic = "{0}{1}{2}".format(directory,
                                                             post["tim"],
                                                             post["ext"])
                                urllib.request.urlretrieve(pic_url, out_pic)
                            except urllib.error.HTTPError as err:
                                pass
                            add_to_downloaded_log(post["tim"], tmp_log)
                            time.sleep(2)
                # Some imageboards allow more than 1 picture per post
                if 'extra_files' in post:
                    for picture in post["extra_files"]:
                        if not was_downloaded(post["tim"], tmp_log):
                            condition_list = []
                            condition_list.append(extension_condition(condition["ext"], post['ext']))
                            condition_list.append(width_condition(condition["width"], post['w']))
                            condition_list.append(height_condition(condition["height"], post['h']))
                            download_img = all_condition_check(condition_list)

                            if download_img:
                                try:
                                    pic_url = ("{0}{1}{2}"
                                               .format(image_url,
                                                       picture["tim"],
                                                       post["ext"]))
                                    out_pic = ("{0}{1}{2}"
                                               .format(directory,
                                                       picture["tim"],
                                                       picture["ext"]))
                                    urllib.request.urlretrieve(pic_url,
                                                               out_pic)
                                except urllib.error.HTTPError as err:
                                    pass
                                add_to_downloaded_log(picture["tim"], tmp_log)
                                time.sleep(2)
            if not is_quiet:
                print('.')
            time.sleep(20)
        except requests.exceptions.HTTPError as err:
            if not is_quiet:
                print('thread 404\'d')
            os.unlink(tmp_log)
            exit(0)



#!/usr/bin/env python3

import time
import json
import os
import re
from scanner import download, chan_info
import subprocess
import urllib.request
import threading


def get_catalog_json(board, chan):
    chan_base_url = chan_info.get_chan_info(chan)[0]
    catalog = urllib.request.urlopen(
              "{0}{1}/catalog.json".format(chan_base_url, board))
    return json.loads(catalog.read().decode("utf8"))


def scan_thread(keyword, catalog_json):
    # Check each thread, threads who contains the keyword are returned
    matched_threads = []
    for i in range(len(catalog_json)):
        for thread in catalog_json[i]["threads"]:
            regex = r'\b{0}\b'.format(keyword)
            # Search thread title
            if 'sub' in thread:
                if re.search(regex, str(thread["sub"]), re.IGNORECASE):
                    matched_threads.append(thread["no"])

            # Search OPs post body
            if 'com' in thread:
                if re.search(regex, str(thread["com"]), re.IGNORECASE):
                    matched_threads.append(thread["no"])

    return matched_threads


def download_thread(thread_id, chan, board, folder, output, condition):
    t = threading.Thread(target=download.download_thread, args=(thread_id,
                                                                board,
                                                                chan,
                                                                output,
                                                                folder,
                                                                True,
                                                                condition))
    t.daemon = True
    t.start()


def add_to_downloaded(id, log_file, output):
    download_log = open("{0}/{1}".format(output, log_file), "a")
    download_log.write("{0}\n".format(id))


def folder_size_mb(folder):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(folder):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += os.path.getsize(fp)
    return total_size / 1000000


def check_quota(folder, quota_mb):
    if int(quota_mb) < folder_size_mb(os.path.join(folder, "downloads")):
        print("Quota limit exceeded. Stopping 4scanner.")
        exit(0)


def scan(keywords_file, output, log_file, quota_mb, wait_time):
    while True:
        if quota_mb:
            check_quota(output, quota_mb)

        curr_time = time.strftime('%d/%b/%Y-%H:%M')
        print("{0} Searching threads...".format(curr_time))

        dl_log = open("{0}/{1}".format(output, log_file), "a")
        dl_log.write(time.strftime('Execution date {0}\n'.format(curr_time)))
        dl_log.close()

        json_file = json.load(open(keywords_file))

        for search in json_file["searches"]:
            if 'imageboard' in search:
                chan = search["imageboard"]
                if not chan_info.get_chan_info(chan):
                    print("{0} is not supported.".format(chan))
                    exit(1)
            else:
                # default
                chan = "4chan"

            # Checking conditions
            condition = {}
            if 'extension' in search:
                condition["ext"] = []
                if isinstance(search['extension'], str):
                    condition["ext"].append(search['extension'])
                else:
                    for extension in search['extension']:
                        condition["ext"].append(extension)

            else:
                condition["ext"] = False

            if 'width' in search:
                condition["width"] = search['width']
            else:
                condition["width"] = False

            if 'height' in search:
                condition["height"] = search['height']
            else:
                condition["height"] = False

            folder_name = search["folder_name"]
            board = search["board"]
            keywords = search["keywords"]
            try:
                catalog_json = get_catalog_json(board, chan)

                for keyword in keywords:
                    threads_id = scan_thread(keyword, catalog_json)

                    dl_log = open("{0}/{1}".format(output, log_file)).read()
                    for thread_id in list(set(threads_id)):
                        if str(thread_id) not in dl_log:
                            download_thread(thread_id, chan, board,
                                            folder_name,
                                            output,
                                            condition)
                            add_to_downloaded(thread_id, log_file, output)
            except urllib.error.HTTPError as err:
                print("Error while opening {0} catalog page. "
                      "Retrying during next scan.".format(board))
                pass

        active_downloads = threading.active_count()-1
        print("{0} threads currently downloading.".format(active_downloads))
        print("Searching again in {0} minutes!"
              .format(str(int(wait_time / 60))))
        time.sleep(wait_time)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='4scanner',
    version='1.2.0',
    description='4chan threads scanner',
    long_description=long_description,
    url='https://github.com/Lacsap-/4scanner',
    author='lacsap',
    author_email='lacsap@cock.li',
    license='MIT',
    scripts=['bin/4downloader', 'bin/4scanner'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='4chan scan download scrape scraper chan imageboard',
    packages=['scanner'],
    install_requires=['requests'],
)
